# core_system/__init__.py
# Peridot Core System Package

# Expose key modules for easier importing
from .command_router import CommandRouter
from .research import MedicalResearchModule

# If you have these files, uncomment them:
# from .ethics import EthicsManager
# from .permissions import PermissionManager
# from .ears import iCouldEars
# from .vision import iCouldVision
